<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2019 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.2/1886137
 */

namespace Redis;

use Application\Config\ConfigManager;
use Application\Factory\InvokableService;
use Interop\Container\ContainerInterface;
use Application\Cache\SimpleCacheDecorator;
use Zend\Cache\StorageFactory;
use Application\Config\ConfigException;
use Zend\Cache\Storage\Adapter\AdapterOptions;

/**
 * Service to handle Redis caching
 *
 * @package Redis
 */
class RedisService extends SimpleCacheDecorator implements InvokableService
{
    const RESOURCE_ID     = 'SwarmRedis';
    const REDIS_NAMESPACE = 'Swarm';
    const REDIS           = 'redis';
    const SEPARATOR       = '^';
    private $resource     = null;
    private $options      = null;

    // The hashing algorithm to use for encoded keys
    const HASHED_KEY_ALGORITHM = "md5";

    /**
     * Create a Redis Service based on settings from the config.
     * Merge the user config into default values.
     *
     * Gives access to the Cache decorator
     *
     * @param ContainerInterface $services
     * @param array|null         $options
     * @throws ConfigException
     */
    public function __construct(ContainerInterface $services, array $options = null)
    {
        $config   = $services->get(ConfigManager::CONFIG);
        $defaults = [
            'name'    => ConfigManager::REDIS,
            'options' => [
                'namespace' => $this->buildNameSpace(
                    ConfigManager::getValue(
                        $config,
                        ConfigManager::REDIS_OPTIONS_NAMESPACE,
                        self::REDIS_NAMESPACE
                    )
                ),
            ]
        ];

        $storage = StorageFactory::factory(
            [
                'adapter' => array_replace_recursive($config[self::REDIS], $defaults),
                'plugins' => [
                    'serializer',
                    'exception_handler' => ['throw_exceptions' => true]
                ],
                'options' => [
                    'resource_id' => self::RESOURCE_ID
                ]
            ]
        );
        parent::__construct($services, $storage);
        $this->resource = $storage->getOptions()->getResourceManager()->getResource(self::RESOURCE_ID);
        $this->options  = $storage->getOptions();
    }

    /**
     * Get the underlying resource that provides redis calls. We will need to use this to get access to calls not
     * provided by the Zend adapter layer.
     * @return mixed
     */
    public function getRedisResource()
    {
        return $this->resource;
    }

    /**
     * Get the underlying Options that can be provided to redis. We can use this to get redis settings and tweak them
     * as required during operations.
     *
     * @return AdapterOptions|null
     */
    public function getRedisOptions()
    {
        return $this->options;
    }

    /**
     * Build name space for redis
     *
     * The namespace can't be longer than 128 character long. This is a limit in the Redis Options Adapter
     *
     * @param string   $prefix   the prefix to use for the namespace.
     * @return string
     */
    public function buildNameSpace($prefix)
    {
        $newNamespace = P4_SERVER_ID ? $prefix . self::SEPARATOR . P4_SERVER_ID : $prefix;
        return $this->validateNamespace($newNamespace);
    }

    /**
     * Get the namespace that this redis cache is using.
     *
     * @return string
     */
    public function getNamespace()
    {
        $cacheOptions = $this->getRedisOptions();
        return $cacheOptions->getNamespace().$cacheOptions->getNamespaceSeparator();
    }
}
